# Generated by roxygen2: do not edit by hand

export(Cassette)
export(cassette_path)
export(cassettes)
export(check_cassette_names)
export(current_cassette)
export(current_cassette_recording)
export(current_cassette_replaying)
export(eject_cassette)
export(insert_cassette)
export(insert_example_cassette)
export(is_recording)
export(is_replaying)
export(local_cassette)
export(local_vcr_configure)
export(local_vcr_configure_log)
export(setup_knitr)
export(skip_if_vcr_off)
export(turn_off)
export(turn_on)
export(turned_off)
export(turned_on)
export(use_cassette)
export(use_vcr)
export(vcr_config_defaults)
export(vcr_configuration)
export(vcr_configure)
export(vcr_configure_log)
export(vcr_configure_reset)
export(vcr_last_request)
export(vcr_last_response)
export(vcr_test_path)
import(rlang)
importFrom(lifecycle,deprecated)
