---
title: "How vcr works, in a lot of details"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{internals}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(vcr)
```

```{r child='../man/rmdhunks/internals-vignette.Rmd', eval=TRUE} 
```



