---
title: "Design of vcr"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Design of vcr}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r child='../man/rmdhunks/vcr-design.Rmd', eval=TRUE} 
```
